/*
 * imflat.c -- Flat field a fits ccd 2D image
 *
 * John Kielkopf
 * kielkopf@louisville.edu
 * 
 *
 * Current Version 1.0 -- March 5, 2005
 *
 *   March 5, 2005 : Version 1.0 : First version from imarith and imstat
 *   
 */

#include <string.h>
#include <stdio.h>
#include "fitsio.h"

int main(int argc, char *argv[])
{
    fitsfile *afptr, *bfptr, *outfptr;  
    int status = 0;  
    int atype, btype, anaxis, bnaxis, check = 1, ii;
    long npixels = 1, firstpix[2] = {1,1}, ntodo;
    long anaxes[2] = {1,1}, bnaxes[2]={1,1};
    long totpix;
    double sum = 0., meanval = 0.;
    double *apix, *bpix;

    if (argc != 4) { 
      printf("Usage: imflat in_image flat_image out_image\n");
      printf("\n");
      printf("Perform flat field correction\n");
      printf("Normalize to the mean value of flat image\n");
      printf("\n");
      printf("Example: \n");
      printf("  imflat image.fits flat.fits image_f.fits\n");
      return(0);
    }
    
    /* Open input images */
    
    fits_open_file(&afptr, argv[1], READONLY, &status); 
    fits_open_file(&bfptr, argv[2], READONLY, &status);

    /* Read dimensions */

    fits_get_img_dim(afptr, &anaxis, &status);  
    fits_get_img_dim(bfptr, &bnaxis, &status);
    fits_get_img_size(afptr, 2, anaxes, &status);
    fits_get_img_size(bfptr, 2, bnaxes, &status);
    
    /* Report errors if any */

    if (status) {
       fits_report_error(stderr, status); 
       return(status);
    }

    /* Check the image sizes */
    
    if ((anaxes[0] != bnaxes[0]) || (anaxes[1] != bnaxes[1])) 
    {
       printf("Error: input images do not have same size\n");
       check = 0;
    }
    else if (anaxis != 2) 
    {
       printf("Error: only  2-D images are supported\n");
       check = 0;    
    }
    

    /* Number of pixels to read in each row */
    
    npixels = anaxes[0];
    
    /* Allocate memory for one row */ 

    apix = (double *) malloc(npixels * sizeof(double)); 
    bpix = (double *) malloc(npixels * sizeof(double)); 

    if (apix == NULL || bpix == NULL) {
      printf("Memory allocation error\n");
      return(1);
    }


    /* Find the mean value for the flat image */
    
    totpix = bnaxes[0] * bnaxes[1];
    if (totpix == 0)
    {
      printf("The flat image is empty\n");
      return(1);
    }
    
    /* Start with first pixel in each row */
    
    firstpix[0] = 1;  

    /* Process flat image one row at a time */
    
    for (firstpix[1] = 1; firstpix[1] <= bnaxes[1]; firstpix[1]++)
    {  
       
      /* Give starting pixel coordinate and number of pixels to read */
       
      if (fits_read_pix(bfptr, TDOUBLE, firstpix, npixels,
           NULL, bpix, NULL, &status))
        break;   

      for (ii = 0; ii < bnaxes[0]; ii++) {
        sum += bpix[ii];                      
      }
    }
    
    meanval = sum / totpix;  
    
    /* Create the new empty output file if the above checks are true */

    if (check && !fits_create_file(&outfptr, argv[3], &status) )
    {

      /* Copy all the header keywords from first image to new output file */
      
      fits_copy_header(afptr, outfptr, &status);

      /* Loop over rows in both images */
        
      for (firstpix[1] = 1; firstpix[1] <= anaxes[1]; firstpix[1]++)
      {
        
        /* Read both images as doubles, regardless of actual datatype */
        /* Give starting pixel coordinate and number of pixels to read */
        /* This version does not support undefined pixels in the image */
        /* Jump out of loop on error */

        if ( (fits_read_pix(afptr, TDOUBLE, firstpix, npixels, 
               NULL, apix, NULL, &status)) 
             || 
             (fits_read_pix(bfptr, TDOUBLE, firstpix, npixels, 
               NULL, bpix, NULL, &status)) )        
          break;   

        /* Perform pixel-by-pixel division */
        /* We expect original CCD frames are unsigned short */
        /* Flat frames are assumed normalized to meanval */

        for(ii=0; ii< npixels; ii++)
        {
          if (bpix[ii] !=0.)
            apix[ii] = meanval*apix[ii]/bpix[ii];
          else
            apix[ii] = 0.;
        }
        
        /* Write new values to output image */

        fits_write_pix(outfptr, TDOUBLE, firstpix, npixels,
                     apix, &status); 
      }
    
      fits_close_file(outfptr, &status);

    }

    free(apix);
    free(bpix);

    fits_close_file(afptr, &status);
    fits_close_file(bfptr, &status);
    
    /* Report any error message */
 
    if (status) fits_report_error(stderr, status); 
    return(status);
}

